package com.example.civilizedcity.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.enums.SqlLike;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.*;

import java.net.URL;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import com.example.civilizedcity.entity.*;
import com.example.civilizedcity.service.*;
import com.example.civilizedcity.vo.IssueExport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;

/**
 * 问题表;(ta_issue)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-12
 */
@Api(tags = "问题表对象功能接口")
@RestController
@RequestMapping("/")
public class TaIssueController extends BaseController {

    @Autowired
    private TaIssueService taIssueService;

    @Autowired
    private TaOrgIssueService taOrgIssueService;

    @Autowired
    private TaAttachService taAttachService;

    @Autowired
    private TaIssueProcessService taIssueProcessService;

    @Autowired
    private SysUserDutyService sysUserDutyService;

    @Autowired
    private SysOrgService sysOrgService;

    @Autowired
    private ExportIssueUtil exportIssueUtil;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taIssue/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable Integer id) throws Exception {

        TaIssue taIssue = taIssueService.getById(id);
        List<TaAttach> attachList = taAttachService.getListBy(Constants.SOURCE_ISSUE, taIssue.getIssueId().toString());
        taIssue.setAttachList(attachList);

        return ResponseBean.success(taIssue);
    }

    /**
     * 构造 查询构造器
     * @param sysUser
     * @param bizStatus
     * @param sourceType
     * @param locId
     * @param mine
     * @param orgId
     * @return
     */
    private QueryWrapper<TaIssue> buildQuery(SysUser sysUser, String issueId, String bizStatus, String sourceType, String locId, Boolean mine, String orgId, Boolean all) {
        QueryWrapper<TaIssue> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotEmpty(issueId), "issue_id", issueId);

        if (Constants.PROCESS_START.equals(bizStatus)) {
            // 待交办
            queryWrapper.likeRight("process_node", bizStatus);
        } else if (Constants.PROCESS_ASSIGNED.equals(bizStatus)) {
            // 已交办
            queryWrapper.likeRight("process_node", bizStatus);
        } else if (Constants.PROCESS_END.equals(bizStatus)) {
            // 已办结
            queryWrapper.eq("process_node", bizStatus);
        } else if ("expired".equals(bizStatus)) {
            // 已逾期
            queryWrapper.ne("process_node", Constants.PROCESS_END);
            queryWrapper.le("expire_date", DateUtils.today());
        } else if ("reject".equals(bizStatus)) {
            // 已打回
            queryWrapper.eq("process_node", Constants.PROCESS_APPLY_REJECT);
            queryWrapper.eq("process_status", Constants.APPLY_PASS);
        } else {
            // 其余节点
            queryWrapper.likeRight(!StringUtils.isEmpty(bizStatus), "process_node", bizStatus);
        }

        if (mine) {
            queryWrapper.eq("create_user", sysUser.getUserId());
        }

        queryWrapper.eq(StringUtils.isNotEmpty(locId), "loc_id", locId);
        queryWrapper.eq(StringUtils.isNotEmpty(orgId), "org_id", orgId);
        queryWrapper.eq(Constants.SOURCE_FEEDBACK.equals(sourceType) && !all, "source_type", Constants.SOURCE_FEEDBACK);
        queryWrapper.ne(!Constants.SOURCE_FEEDBACK.equals(sourceType) && !all, "source_type", Constants.SOURCE_FEEDBACK);
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByDesc("create_date");

        return queryWrapper;
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taIssue")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                             @ApiParam("问题单ID") @RequestParam(value = "issueId", required = false) String issueId,
                             @ApiParam("问题单状态") @RequestParam(value = "bizStatus", required = false) String bizStatus,
                             @ApiParam("来源") @RequestParam(value = "sourceType", required = false) String sourceType,
                             @ApiParam("点位") @RequestParam(value = "locId", required = false) String locId,
                             @ApiParam("我的") @RequestParam(value = "mine", defaultValue = "false") Boolean mine,
                             @ApiParam("责任单位") @RequestParam(value = "orgId", required = false) String orgId,
                             @ApiParam("是否全部") @RequestParam(value = "all", defaultValue = "false") Boolean all) throws Exception {

        SysUser sysUser = currentUser();
        boolean isRooter = Constants.ROOT_ID.equals(sysUser.getUserId());

        List<String> dutyList = sysUserDutyService.getListByUser(sysUser.getUserId());
        if (!isRooter && (dutyList == null || dutyList.size() < 1)) {
            return ResponseBean.error("暂无权限查询数据");
        }

        IPage<TaIssue> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaIssue> queryWrapper = buildQuery(sysUser, issueId, bizStatus, sourceType, locId, mine, orgId, all);
        IPage<TaIssue> result = taIssueService.page(pg, queryWrapper);

        return ResponseBean.success(result);
    }


    /**
     * 导出
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("导出")
    @PostMapping("/taIssue/export")
    public ResponseBean export(@ApiParam("问题单ID") @RequestParam(value = "issueId", required = false) String issueId,
                               @ApiParam("问题单状态") @RequestParam(value = "bizStatus", required = false) String bizStatus,
                               @ApiParam("来源") @RequestParam(value = "sourceType", required = false) String sourceType,
                               @ApiParam("点位") @RequestParam(value = "locId", required = false) String locId,
                               @ApiParam("我的") @RequestParam(value = "mine", defaultValue = "false") Boolean mine,
                               @ApiParam("责任单位") @RequestParam(value = "orgId", required = false) String orgId,
                               @ApiParam("是否全部") @RequestParam(value = "all", defaultValue = "false") Boolean all,
                               HttpServletResponse response) throws Exception {

        SysUser sysUser = currentUser();
        boolean isRooter = Constants.ROOT_ID.equals(sysUser.getUserId());

        List<String> dutyList = sysUserDutyService.getListByUser(sysUser.getUserId());
        if (!isRooter && (dutyList == null || dutyList.size() < 1)) {
            return ResponseBean.error("暂无权限查询数据");
        }

        QueryWrapper<TaIssue> queryWrapper = buildQuery(sysUser, issueId, bizStatus, sourceType, locId, mine, orgId, all);
        List<TaIssue> list = taIssueService.list(queryWrapper);

        List<IssueExport> exportList = exportIssueUtil.getExportList(list);
        ExcelUtils.flush(response, IssueExport.class, exportList, "问题单列表");

        return null;
    }

    /**
     * 新增数据
     *
     * @param taIssue 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taIssue")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaIssue taIssue) throws Exception {
        // 督察员上报专用

        if (null == taIssue.getAttachList() || taIssue.getAttachList().size() == 0) {
            return ResponseBean.error("请上传照片");
        }

        SysUser user = currentUser();
        taIssueService.createNew(taIssue, user);
        return ResponseBean.success(taIssue);
    }

    /**
     * 更新数据
     *
     * @param taIssue 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/taIssue/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaIssue taIssue,
                             @ApiParam("对象ID") @PathVariable Integer id) throws Exception {

        TaIssue origin = taIssueService.getById(id);
        if (null == origin || origin.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到原始问题单");
        }

        List<TaAttach> attachList = taIssue.getAttachList();
        if (null == attachList || attachList.size() == 0) {
            return ResponseBean.error("拍照或者视频没有上传");
        }

        taIssueService.updateIssue(taIssue, origin);
        return ResponseBean.success(origin);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @SaCheckPermission("manager")
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taIssue/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable Integer id) throws Exception {
//        taIssueService.removeLogicById(id);

        TaIssue taIssue = taIssueService.getById(id);
        if (null == taIssue || taIssue.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到问题");
        }

        taIssue.setUpdateUser(currentUser().getUserId());
        taIssue.setUpdateDate(LocalDateTime.now());
        taIssue.setStatus(Constants.STATUS_DELETE);

        taIssueService.updateById(taIssue);
        return ResponseBean.success("success");
    }
}